@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005-2006 Alex Murray <pragmatine@gmail.com>
+ * Copyright (C) 2005-2009 Alex Murray <murray.alex@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,6 +44,8 @@
 
 const gchar *plugin_name = "libsensors";
 
+GHashTable *hash_table = NULL;
+
 enum {
 	LIBSENSORS_CHIP_PARSE_ERROR,
 	LIBSENSORS_MISSING_FEATURE_ERROR,
@@ -56,7 +58,7 @@ enum {
 #define LIBSENSORS_ALTERNATIVE_CONFIG_FILE "/usr/local/etc/sensors.conf"
 #endif
 
-static regex_t uri_re;
+regex_t uri_re;
 
 static char *get_chip_name_string(const sensors_chip_name *chip) {
 	char *name;
@@ -223,6 +225,7 @@ static void check_sensor_with_data(GList **sensors,
                                         
                         }
 
+                        g_hash_table_insert(hash_table, g_strdup(url), (void *)chip);
                         // the id identifies a particular sensor for the user;
                         // we default to the label returned by libsensors
                         sensors_applet_plugin_add_sensor_with_limits(sensors,
@@ -258,7 +261,8 @@ static GList *libsensors_plugin_get_sensors(void) {
 	 * file - if neither succeed, exit */
 	if ((file = fopen (LIBSENSORS_CONFIG_FILE, "r")) == NULL) {
 		if ((file = fopen (LIBSENSORS_ALTERNATIVE_CONFIG_FILE, "r")) == NULL) {
-			return;
+                        g_debug("%s: error opening libsensors config file... ", __FUNCTION__);
+			return sensors;
 		}
 	}
 	
@@ -266,7 +270,8 @@ static GList *libsensors_plugin_get_sensors(void) {
 	 * valid, close file and return */
 	if (sensors_init(file) != 0) {
 		fclose(file);
-		return;
+                g_debug("%s: error initing libsensors from config file...", __FUNCTION__);
+		return sensors;
 	}
 	fclose(file);
 
@@ -293,7 +298,8 @@ static GList *libsensors_plugin_get_sensors(void) {
         
         int nr = 0;
         if (sensors_init(NULL) != 0) {
-                return;
+                g_debug("%s: error initing libsensors", __FUNCTION__);
+                return sensors;
         }
 	i = 0;
 	while ((chip_name = sensors_get_detected_chips(NULL, &nr)))
@@ -369,7 +375,7 @@ static GList *libsensors_plugin_get_sensors(void) {
 			if (!input_feature)
                         {
                                 g_debug("%s: %d: could not get input subfeature for: %s\n",
-                                        __FILE__, __LINE__, chip_name);
+                                        __FILE__, __LINE__, chip_name_string);
 				continue;
                         }
                         // if still here we got input feature so get label
@@ -410,6 +416,7 @@ static GList *libsensors_plugin_get_sensors(void) {
                                   (type == VOLTAGE_SENSOR ? "voltage" : "error"))), label, value);
 
                         path = g_strdup_printf ("sensor://%s/%d", chip_name_string, input_feature->number);
+			g_hash_table_insert(hash_table, g_strdup(path), (void *)chip_name);
                         sensors_applet_plugin_add_sensor_with_limits(&sensors,
                                                                      path,
                                                                      label,
@@ -431,80 +438,61 @@ static GList *libsensors_plugin_get_sensors(void) {
 }
 
 static gdouble libsensors_plugin_get_sensor_value(const gchar *path, 
-                                                             const gchar *id, 
-                                                             SensorType type,
-                                                             GError **error) {
+						  const gchar *id, 
+						  SensorType type,
+						  GError **error) {
 	gdouble result = 0;
 	regmatch_t m[3];
 
 	/* parse the uri into a (chip, feature) tuplet */
 	if (regexec (&uri_re, path, 3, m, 0) == 0) {
-		char *desired_chip_s;
-		sensors_chip_name desired_chip;
-		int feature;
-
-		int i;
 		const sensors_chip_name *found_chip;
+		int feature;
+		int i;
 
-		desired_chip_s = g_strndup (path + m[1].rm_so, m[1].rm_eo - m[1].rm_so);
-		if (sensors_parse_chip_name (desired_chip_s, &desired_chip) != 0)
-                {
-			g_set_error (error, SENSORS_APPLET_PLUGIN_ERROR, LIBSENSORS_CHIP_PARSE_ERROR, "Error parsing chip name");
-                } else {
+		if ((found_chip = g_hash_table_lookup(hash_table,
+						     path)) != NULL)
+		{
+			gdouble value;
 			feature = atoi(path + m[2].rm_so);
 #if SENSORS_API_VERSION < 0x400
-                        /* search for the correct chip */
-			for (i = 0; (found_chip = sensors_get_detected_chips (&i)); ) {
-				if (sensors_match_chip (desired_chip, *found_chip)) {
-                                        double value;
-					/* retrieve the value of the feature */
-					if (sensors_get_feature (*found_chip, feature, &value) == 0) {
-						result = value;
-                                        } else {
-						g_set_error (error, SENSORS_APPLET_PLUGIN_ERROR, LIBSENSORS_MISSING_FEATURE_ERROR, "Error retrieving sensor value");
-                                        }
-                                        break;
-                                }
-                        }
-                        // work around mem-leak in libsensors3
-                        if (desired_chip.bus == SENSORS_CHIP_NAME_BUS_DUMMY)
-                        {
-                                g_assert(desired_chip.busname != NULL);
-                                free(desired_chip.busname);
-                        }
-                        free(desired_chip.prefix);
-
+			/* retrieve the value of the feature */
+			if (sensors_get_feature (*found_chip, feature, &value) == 0) {
+				result = value;
+			} else {
+				g_set_error (error, SENSORS_APPLET_PLUGIN_ERROR, LIBSENSORS_MISSING_FEATURE_ERROR, "Error retrieving sensor value");
+			}
 #else
-                        i = 0;
-                        if (found_chip = sensors_get_detected_chips(&desired_chip, &i)) {
-                                gdouble value;
-                                
-                                if (sensors_get_value(found_chip, feature, &value) >= 0) {
-                                        result = value;
-                                } else {
-                                        g_set_error (error, SENSORS_APPLET_PLUGIN_ERROR, LIBSENSORS_MISSING_FEATURE_ERROR, "Error retrieving sensor value");
-                                }
+			if (sensors_get_value(found_chip, feature, &value) >= 0) {
+				result = value;
+			} else {
+				g_set_error (error, SENSORS_APPLET_PLUGIN_ERROR, LIBSENSORS_MISSING_FEATURE_ERROR, "Error retrieving sensor value");
 			}
 #endif
-			if (found_chip == NULL)
-				g_set_error (error, SENSORS_APPLET_PLUGIN_ERROR, LIBSENSORS_CHIP_NOT_FOUND_ERROR, "Chip not found");
+		} else {
+			g_set_error (error, SENSORS_APPLET_PLUGIN_ERROR, LIBSENSORS_CHIP_NOT_FOUND_ERROR, "Chip not found");
 		}
-		g_free (desired_chip_s);
-	} else
+	} else {
 		g_set_error (error, SENSORS_APPLET_PLUGIN_ERROR, LIBSENSORS_REGEX_URL_COMPILE_ERROR, "Error compiling URL regex");
+	}
 	return result;
-        
 }
 
 
 static GList *libsensors_plugin_init() {
 	/* compile the regular expressions */
 	if (regcomp(&uri_re, "^sensor://([a-z0-9-]+)/([0-9]+)$", 
-                    REG_EXTENDED | REG_ICASE) != 0) {
-                g_debug("Error compiling regexp...not initing libsensors sensors interface");
-                return;
-        }
-        
+		    REG_EXTENDED | REG_ICASE) != 0) {
+		g_debug("Error compiling regexp...not initing libsensors sensors interface");
+		return NULL;
+	}
+
+	/* create hash table to associate path strings with sensors_chip_name
+	 * pointers - make sure it free's the keys strings on destroy */
+	hash_table = g_hash_table_new_full(g_str_hash,
+					   g_str_equal,
+					   g_free,
+					   NULL);
 	return libsensors_plugin_get_sensors();
 }
 
