@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005-2006 Alex Murray <pragmatine@gmail.com>
+ * Copyright (C) 2005-2009 Alex Murray <murray.alex@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
 
 static const gchar * const compatible_versions[] = {
         PACKAGE_VERSION, /* always list current version */
-        "2.2.0"
+	"2.2.2",
 };
 
 #define NUM_COMPATIBLE_VERSIONS G_N_ELEMENTS(compatible_versions)
