@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005-2006 Alex Murray <pragmatine@gmail.com>
+ * Copyright (C) 2005-2009 Alex Murray <murray.alex@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -1205,6 +1205,13 @@ static ActiveSensor *sensors_applet_find_active_sensor(SensorsApplet *sensors_ap
 void sensors_applet_display_layout_changed(SensorsApplet *sensors_applet) {
         /* update sensors since will need to update icons / graphs etc
          * if weren't displayed before */
+        GList *list = NULL;
+        for (list = sensors_applet->active_sensors;
+             list != NULL;
+             list = list->next) {
+                ActiveSensor *as = (ActiveSensor *)list->data;
+                as->updated = FALSE;
+        }
         sensors_applet_update_active_sensors(sensors_applet);
 	sensors_applet_pack_display(sensors_applet);
 }
